"use strict";

function _toConsumableArray(arr) { return _arrayWithoutHoles(arr) || _iterableToArray(arr) || _unsupportedIterableToArray(arr) || _nonIterableSpread(); }

function _nonIterableSpread() { throw new TypeError("Invalid attempt to spread non-iterable instance.\nIn order to be iterable, non-array objects must have a [Symbol.iterator]() method."); }

function _unsupportedIterableToArray(o, minLen) { if (!o) return; if (typeof o === "string") return _arrayLikeToArray(o, minLen); var n = Object.prototype.toString.call(o).slice(8, -1); if (n === "Object" && o.constructor) n = o.constructor.name; if (n === "Map" || n === "Set") return Array.from(o); if (n === "Arguments" || /^(?:Ui|I)nt(?:8|16|32)(?:Clamped)?Array$/.test(n)) return _arrayLikeToArray(o, minLen); }

function _iterableToArray(iter) { if (typeof Symbol !== "undefined" && Symbol.iterator in Object(iter)) return Array.from(iter); }

function _arrayWithoutHoles(arr) { if (Array.isArray(arr)) return _arrayLikeToArray(arr); }

function _arrayLikeToArray(arr, len) { if (len == null || len > arr.length) len = arr.length; for (var i = 0, arr2 = new Array(len); i < len; i++) { arr2[i] = arr[i]; } return arr2; }

module.exports = function babelPluginDedent(babel) {
  var t = babel.types;
  return {
    visitor: {
      CallExpression: function CallExpression(path) {
        var node = path.node;

        if (t.isIdentifier(node.callee, {
          name: 'dedent'
        })) {
          if (t.isTemplateLiteral(node.arguments[0])) {
            transform(node.arguments[0].quasis);
            path.replaceWith(node.arguments[0]);
          } else if (t.isTaggedTemplateExpression(node.arguments[0])) {
            transform(node.arguments[0].quasi.quasis);
            path.replaceWith(node.arguments[0]);
          }
        }
      },
      TaggedTemplateExpression: function TaggedTemplateExpression(path) {
        var node = path.node;

        if (t.isIdentifier(node.tag, {
          name: 'dedent'
        })) {
          transform(node.quasi.quasis);
          path.replaceWith(node.quasi);
        }
      }
    }
  };
};

function transform(quasis) {
  var elements = quasis.filter(function (element) {
    return element.type === 'TemplateElement';
  });
  var matches = [];
  rtrim(elements[elements.length - 1]);
  elements.forEach(function (element) {
    var match;

    if (match = element.value.raw.match(/\n[\t ]+/g)) {
      matches.push.apply(matches, _toConsumableArray(match));
    }
  });

  if (matches.length) {
    var size = Math.min.apply(Math, _toConsumableArray(matches.map(function (value) {
      return value.length - 1;
    })));
    var pattern = new RegExp("\n[\t ]{".concat(size, "}"), 'g');
    ['raw', 'cooked'].forEach(function (type) {
      elements.forEach(function (element) {
        element.value[type] = element.value[type].replace(pattern, '\n');
      });
    });
  }

  ltrim(elements[0]);
}

function ltrim(element) {
  var pattern = /^\r?\n/;

  if (pattern.test(element.value.raw)) {
    element.value.raw = element.value.raw.replace(pattern, '');
    element.value.cooked = element.value.cooked.replace(pattern, '');
  }

  return element;
}

function rtrim(element) {
  var match;

  if (match = element.value.raw.match(/\r?\n([\t ]*)$/)) {
    var pattern = new RegExp("\r?\n[\t ]{".concat(match[1].length, "}$"));
    element.value.raw = element.value.raw.replace(pattern, '');
    element.value.cooked = element.value.cooked.replace(pattern, '');
  }

  return element;
}